%Yann-Al Le Borgne - October 2005 -
%www.ulb.ac.be/di/map/yleborgn/sensorNet

function [Gfin] = main(output)
% output is the name of the output files.

%Get environment design
[U0,a,CMedium,CBorder,CSource,d,f,b,gd,sf,ns]=getPDESpec();
sizeBackground=size(gd,2)

c=CMedium
for i=2:sizeBackground
    c=[c '!' CBorder];
    b2=[0 1 1 1 1 1 48 48 49 48]';
    b=[b b2 b2 b2 b2];
end
%sensorPos: matrix of size nbSensors*2
[sensorPos]=getSensorsSpec();
nbSensors=size(sensorPos,1);

[duration nbHeatSources radius, heatPos, heatIntensity]=getHeatSpec()
%Duration: nb of different moves
%nbHeatSources: Nb of different heat sources
%radius: radius of heat sources
%heatPos: matrix of size duration*nbHeatSources*2
%heatIntensity: Matrix of size duration*nbHeatSources


%Add heat sources to the environment
for i=1:nbHeatSources
    gd(:,i+sizeBackground)=[4 heatPos(1,i,1) heatPos(1,i,2) radius radius 0 0 0 0 0];
    ns(:,i+sizeBackground)=[69 48+i];
    sf=[sf '+E' int2str(i)];
    b2=[0 1 1 1 1 1 48 48 49 48]';
    b=[b b2 b2 b2 b2];
    
    a=[a '!0.0'];
    c=[c '!' CSource];
    d=[d '!1.0'];
    f=[f '!' int2str(heatIntensity(1,i))];
end

%Grain of the simulation
tick=0.1;

%Data read by sensors will be stored there
resultat=zeros(nbSensors,duration/tick);

for i=1:duration
    display(['Step:' int2str(i)])

    for k=(sizeBackground+1):(nbHeatSources+sizeBackground)
        gd(2,k)=heatPos(i,k-sizeBackground,1);
        gd(3,k)=heatPos(i,k-sizeBackground,2);
    end
    
    [g bt]=decsg(gd);

    f='0';
    c=CMedium;
    for k=2:(nbHeatSources+sizeBackground)
        [Y I]=max(bt(k,2:(nbHeatSources+sizeBackground)));
        if (I>(sizeBackground-1)) 
           f=[f '!' int2str(heatIntensity(i,I-sizeBackground+1))];
           c=[c '!' CSource];
        else 
           f=[f '!0'];
           c=[c '!' CBorder];
        end
    end

    if (i==1) [p,e,t]=initmesh(g);
    else 
        [p2,e2,t2]=initmesh(g);
       
        nbC=size(p2,2); %nbC = nb of vertices in the mesh
        Uadapt=0;
        %This is used to get initial temp conditions for next heat sources
        %positionning
        %moved
        for l=1:nbC
            Uadapt(1,l)=tri2grid(p,t,resultatTemp(:,size(resultatTemp,2)),p2(1,l),p2(2,l)); 
        end
        U0=Uadapt;
        p=p2;e=e2;t=t2;
    end
    pdemesh(p,e,t); %Display mesh
    pause(1);
    
    %Get data

    timeSeq=[(i-1):tick:(i-tick)];
    resultatTemp = parabolic(U0,timeSeq,b,p,e,t,c,a,f,d); 
    videoPDE{i}=resultatTemp;
    for k=1:length(timeSeq)
        for l=1:nbSensors
         resultat(l,(i-1)*length(timeSeq)+k)=tri2grid(p,t,resultatTemp(:,k),sensorPos(l,1),sensorPos(l,2)); 
        end
    end
    %Save P,E and T for generating a video afterwards
    P{i}=p;
    E{i}=e;
    T{i}=t;

end

%Generate some nice videos
videoGen(output,videoPDE,E,P,T,duration,tick)
sensorData=resultat
%Save data
save output sensorData sensorPos;
save 'sensorData.txt' -ASCII sensorData;
save 'sensorPos.txt' -ASCII sensorPos;
heatPosY=heatPos(:,:,2);
save 'heatPosY.txt' -ASCII heatPosY;
heatPosX=heatPos(:,:,1);
save 'heatPosX.txt' -ASCII heatPosX;
display('-------- Done ----------');
